# Modbus Library for Arduino
Allows your Arduino to communicate via Modbus protocol

[![GitHub release (latest by date including pre-releases)](https://img.shields.io/github/v/release/epsilonrt/modbus-arduino?include_prereleases)](https://github.com/epsilonrt/modbus-arduino/releases) 
[![PlatformIO Registry](https://badges.registry.platformio.org/packages/epsilonrt/library/modbus-arduino.svg)](https://registry.platformio.org/libraries/epsilonrt/modbus-arduino) 
[![Arduino Registry](https://www.ardu-badge.com/badge/Modbus-Arduino.svg)](https://www.arduinolibraries.info/libraries/modbus-arduino)

[![Framework](https://img.shields.io/badge/Framework-Arduino-blue)](https://www.arduino.cc/)
[![Platform ATMELAVR](https://img.shields.io/badge/Platform-AtmelAVR-blue)](#)
[![Platform ATMELSAM](https://img.shields.io/badge/Platform-AtmelSAM-blue)](#)
[![Platform Teensy](https://img.shields.io/badge/Platform-Teensy-green)](#)
[![Platform ESP8266](https://img.shields.io/badge/Platform-Espressif8266-orange)](#)
[![Platform ESP32](https://img.shields.io/badge/Platform-Espressif32-orange)](#)

---

<a href="https://modbus.org/">
  <img src="https://github.com/epsilonrt/modbus-arduino/raw/master/extras/modbus.png" alt="Modbus Logo" align="right" valign="top">
</a>

The Modbus is a master-slave protocol used in industrial automation and can be used in other areas, such as home automation.

The Modbus generally uses serial RS-232 or RS-485 as physical layer (then called Modbus Serial) and
TCP/IP via Ethernet or WiFi (Modbus TCP).

In the current version the library allows the Arduino operate **as a slave**, supporting Modbus Serial and
Modbus over IP. For more information about Modbus see:

* [Wikipedia article](https://en.wikipedia.org/wiki/Modbus)  
* [MODBUS Application Protocol Specification](http://www.modbus.org/docs/Modbus_Application_Protocol_V1_1b.pdf)  
* [MODBUS Messaging on TCP/IP Implementation Guide](http://www.modbus.org/docs/Modbus_Messaging_Implementation_Guide_V1_0b.pdf)  
* [MODBUS over serial line specification and implementation guide ](http://www.modbus.org/docs/Modbus_over_serial_line_V1_02.pdf)

**Author's note (motivation and thanks):**

It all started when I found the Modbus RTU Arduino library of Juan Pablo Zometa. I had extend the
library to support other Modbus functions.

After researching several other Modbus libraries I realized strengths and weaknesses in all of them.
I also thought it would be cool have a base library for Modbus and derive it for each type of physical layer used.

I appreciate the work of all the authors of the other libraries, of which I used several ideas to compose the modbus-arduino.
At the end of this document is a list of libraries and their authors.

## Features

* Operates as a slave (master mode in development)   
* Supports Modbus over Serial line (RS-232 or RS485) and Modbus TCP
* Reply exception messages for all supported functions  
* Modbus functions supported:  
    * 0x01 - Read Coils  
    * 0x02 - Read Input Status (Read Discrete Inputs)  
    * 0x03 - Read Holding Registers  
    * 0x04 - Read Input Registers  
    * 0x05 - Write Single Coil  
    * 0x06 - Write Single Register  
    * 0x0F - Write Multiple Coils  
    * 0x10 - Write Multiple Registers  
    * 0x11 - Report Server ID

**Notes:**

1. The offsets for registers are 0-based. So be careful when setting your supervisory system or your testing software. For example, in ScadaBR (http://www.scadabr.com.br)
offsets are 0-based, then, a register configured as 100 in the library is set to 100 in ScadaBR. On the other hand, in the CAS Modbus Scanner
(http://www.chipkin.com/products/software/modbus-software/cas-modbus-scanner/) offsets are 1-based, so a register configured as 100 in library should be 101 in this software.

2. Early in the library Modbus.h file there is an option to limit the operation
to the functions of Holding Registers, saving space in the program memory.
Just comment out the following line:

		#define USE_HOLDING_REGISTERS_ONLY

Thus, only the following functions are supported:

* 0x03 - Read Holding Registers  
* 0x06 - Write Single Register  
* 0x10 - Write Multiple Registers  

## How to

There are four classes corresponding to five headers that may be used:

* Modbus-Arduino - Base Library  (this repository)  
* [Modbus-Serial](https://github.com/epsilonrt/modbus-serial) - Modbus Serial RTU Library    
* [Modbus-Ethernet](https://github.com/epsilonrt/modbus-ethernet) - Modbus TCP Library (standard Ethernet Shield)   
* [Modbus-EtherCard](https://github.com/epsilonrt/modbus-ethercard) - Modbus TCP Library (for ENC28J60 chip)  
* [Modbus-Esp8266AT](https://github.com/epsilonrt/modbus-esp8266at) - Modbus IP Library (for ESP8266 chip with AT firmware)   

By opting for Modbus Serial or Modbus TCP you must include in your sketch the corresponding header and the base library header, eg:

    #include <Modbus.h>
    #include <ModbusSerial.h>


## Modbus Jargon

In this library was decided to use the terms used in Modbus to the methods names, then is important clarify the names of
register types:

| Register type        | Use as             | Access            | Library methods               |
| -------------------- | ------------------ | ----------------- | ----------------------------- |
| Coil                 | Digital Output     | Read/Write        | addCoil(), coil(), setCoil()  |
| Holding Register     | Analog Output      | Read/Write        | addHreg(), hreg(), setHreg()  |
| Input Status         | Digital Input      | Read Only         | addIsts(), ists(), setIsts()  |
| Input Register       | Analog Input       | Read Only         | addIreg(), ireg(), setIreg()  |

**Notes:**

1. _Input Status_ is sometimes called _Discrete Input_.  
2. _Holding Register_ or just _Register_ is also used to store values in the slave.  
3. Examples of use: A _Coil_ can be used to drive a lamp or LED. A _Holding Register_ to
store a counter or drive a Servo Motor. A _Input Status_ can be used with a reed switch
in a door sensor and a _Input Register_ with a temperature sensor.

License
=======
The code in this repo is licensed under the BSD New License. See LICENSE for more info.

